# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import errno
import socket
import threading
import time

import six

from driverng.transport.base import BaseTransport, TransportException

if six.PY3:
    from http.server import BaseHTTPRequestHandler, HTTPServer
else:
    from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer

filename_received = {}
filename_waiting = {}
client_reponses = {}
nb_posts = 0
der_posts = 0
logger = None


class HTTPHandler(BaseHTTPRequestHandler):
    def get_filename(self):
        return "resu_c_aqua-%s-%s.xml" % (
            self.client_address[1],
            datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
        )

    def _set_headers(self):
        self.send_response(200)
        self.send_header("Content-type", "text/xml")
        #  self.end_headers() not supported by Aqua client!

    def do_GET(self):
        self._set_headers()
        self.wfile.write("<html><body><h1>Get not implemented!</h1></body></html>")

    def do_HEAD(self):
        self._set_headers()

    def do_POST(self):
        global nb_posts, der_post
        nb_posts = nb_posts + 1
        der_post = time.time()
        logger.info("POSTS nb_posts :%d", nb_posts)
        content_length = int(
            self.headers["Content-Length"]
        )  # <--- Gets the size of data
        post_data = self.rfile.read(content_length)  # <--- Gets the data itself
        filename = self.get_filename()
        filename_received[filename] = (post_data, self)

        self._set_headers()
        for i in range(300):  # 30s pour recevoir la reponse, timeout Aqua
            reponse = client_reponses.get(self, None)
            if reponse:
                self.wfile.write(reponse)
                del client_reponses[self]
                return
            else:
                time.sleep(0.1)

    def finish(self):
        if client_reponses.get(self, None):
            del client_reponses[self]
        logger.info(
            " nb_posts:%d finish:%s, received :%d, waiting:%d",
            nb_posts,
            self.client_address,
            len(filename_received),
            len(filename_waiting),
        )


class MyHTTPServer(HTTPServer):

    request_queue_size = 200  # nombre max de clients en attente


class HTTPTransport(BaseTransport):
    def __init__(self, host=None, port=None, mode="server", **kwargs):
        super(HTTPTransport, self).__init__(**kwargs)
        global logger
        if host is None:
            raise TransportException(
                "L'adresse ip et le port http ne sont pas paramétrés sur la connexion"
            )
        if port is None:
            host, port = host.rsplit(":", 1)
            port = int(port)
        self.host = host
        self.port = port
        logger = self.logger

        if mode == "server":
            self.httpd = MyHTTPServer((host, port), HTTPHandler)
            self.logger.info("Starting httpd...")
            t = threading.Thread(target=self.httpd.serve_forever)
            t.daemon = True
            t.start()
            self.thread = t
        else:  # client

            self.logger.info("Try to connect to %s:%d", host, port)
            raise Exception("transport Http client non-implémenté")

        self.logger.info("Starting httpd OK")

    def set_timeout(self, timeout):
        pass

    def poll(self, timeout, wqueue):
        if filename_received:
            return True
        if nb_posts > 5 and (time.time() - der_post) > 120:
            self.restart()
        return False

    def pre_send(self, meta=None):
        self.meta = meta

    def _read(self, size):
        filename, value = filename_received.items()[0]
        self.filename = filename
        self.client = value[1]
        del filename_received[filename]
        filename_waiting[filename] = value
        client_reponses.setdefault(self.client, None)

        return value[0]

    def get_recv_filename(self):
        return self.filename

    def _write(self, data):
        filename = self.meta.get("filename", None)
        if not filename:  # reponse automatique à la derniere trame
            del filename_waiting[self.filename]
            client_reponses[self.client] = data
            return
        values = filename_waiting.get(filename, None)
        if values:
            client_reponses[values[1]] = data  # envoie au bon client
            del filename_waiting[filename]
        else:
            self.logger.error("httpd _write, no handler client found")
        return

    def restart(self):
        global nb_posts, der_post
        global filename_received, filename_waiting, client_reponses
        nb_posts = 0
        der_post = time.time()
        self.httpd.shutdown()
        self.httpd.server_close()
        self.thread.join()
        self.httpd = MyHTTPServer((self.host, self.port), HTTPHandler, self.logger)
        t = threading.Thread(target=self.httpd.serve_forever)
        t.daemon = True
        t.start()
        self.thread = t
        filename_received = {}
        filename_waiting = {}
        client_reponses = {}
        self.logger.info("re Starting httpd OK")

    def close(self):
        self.logger.info("Stopping httpd ")
        try:
            self.httpd.shutdown()
            self.httpd.close()
        except socket.error as e:
            if e.args[0] not in (errno.ENOTCONN, errno.EBADF):
                raise
